@@ -8,6 +8,11 @@ use super::*;
 /// The parser parses both values and expressions into `Expression`s.
 #[derive(PartialEq, Debug, Clone)]
 pub(crate) enum Expression<'src> {
+  /// `lhs && rhs`
+  And {
+    lhs: Box<Expression<'src>>,
+    rhs: Box<Expression<'src>>,
+  },
   /// `assert(condition, error)`
   Assert {
     condition: Condition<'src>,
@@ -38,6 +43,11 @@ pub(crate) enum Expression<'src> {
     lhs: Option<Box<Expression<'src>>>,
     rhs: Box<Expression<'src>>,
   },
+  /// `lhs || rhs`
+  Or {
+    lhs: Box<Expression<'src>>,
+    rhs: Box<Expression<'src>>,
+  },
   /// `"string_literal"` or `'string_literal'`
   StringLiteral { string_literal: StringLiteral<'src> },
   /// `variable`
@@ -51,25 +61,27 @@ impl<'src> Expression<'src> {
 }
 
 impl<'src> Display for Expression<'src> {
-  fn fmt(&self, f: &mut Formatter) -> Result<(), fmt::Error> {
+  fn fmt(&self, f: &mut Formatter) -> fmt::Result {
     match self {
+      Self::And { lhs, rhs } => write!(f, "{lhs} && {rhs}"),
       Self::Assert { condition, error } => write!(f, "assert({condition}, {error})"),
       Self::Backtick { token, .. } => write!(f, "{}", token.lexeme()),
-      Self::Join { lhs: None, rhs } => write!(f, "/ {rhs}"),
-      Self::Join {
-        lhs: Some(lhs),
-        rhs,
-      } => write!(f, "{lhs} / {rhs}"),
+      Self::Call { thunk } => write!(f, "{thunk}"),
       Self::Concatenation { lhs, rhs } => write!(f, "{lhs} + {rhs}"),
       Self::Conditional {
         condition,
         then,
         otherwise,
       } => write!(f, "if {condition} {{ {then} }} else {{ {otherwise} }}"),
+      Self::Group { contents } => write!(f, "({contents})"),
+      Self::Join { lhs: None, rhs } => write!(f, "/ {rhs}"),
+      Self::Join {
+        lhs: Some(lhs),
+        rhs,
+      } => write!(f, "{lhs} / {rhs}"),
+      Self::Or { lhs, rhs } => write!(f, "{lhs} || {rhs}"),
       Self::StringLiteral { string_literal } => write!(f, "{string_literal}"),
       Self::Variable { name } => write!(f, "{}", name.lexeme()),
-      Self::Call { thunk } => write!(f, "{thunk}"),
-      Self::Group { contents } => write!(f, "({contents})"),
     }
   }
 }
@@ -80,6 +92,13 @@ impl<'src> Serialize for Expression<'src> {
     S: Serializer,
   {
     match self {
+      Self::And { lhs, rhs } => {
+        let mut seq = serializer.serialize_seq(None)?;
+        seq.serialize_element("and")?;
+        seq.serialize_element(lhs)?;
+        seq.serialize_element(rhs)?;
+        seq.end()
+      }
       Self::Assert { condition, error } => {
         let mut seq: <S as Serializer>::SerializeSeq = serializer.serialize_seq(None)?;
         seq.serialize_element("assert")?;
@@ -101,13 +120,6 @@ impl<'src> Serialize for Expression<'src> {
         seq.serialize_element(rhs)?;
         seq.end()
       }
-      Self::Join { lhs, rhs } => {
-        let mut seq = serializer.serialize_seq(None)?;
-        seq.serialize_element("join")?;
-        seq.serialize_element(lhs)?;
-        seq.serialize_element(rhs)?;
-        seq.end()
-      }
       Self::Conditional {
         condition,
         then,
@@ -121,6 +133,20 @@ impl<'src> Serialize for Expression<'src> {
         seq.end()
       }
       Self::Group { contents } => contents.serialize(serializer),
+      Self::Join { lhs, rhs } => {
+        let mut seq = serializer.serialize_seq(None)?;
+        seq.serialize_element("join")?;
+        seq.serialize_element(lhs)?;
+        seq.serialize_element(rhs)?;
+        seq.end()
+      }
+      Self::Or { lhs, rhs } => {
+        let mut seq = serializer.serialize_seq(None)?;
+        seq.serialize_element("or")?;
+        seq.serialize_element(lhs)?;
+        seq.serialize_element(rhs)?;
+        seq.end()
+      }
       Self::StringLiteral { string_literal } => string_literal.serialize(serializer),
       Self::Variable { name } => {
         let mut seq = serializer.serialize_seq(None)?;
