@@ -34,11 +34,10 @@ pub(crate) use {
   },
   camino::Utf8Path,
   clap::ValueEnum,
-  derivative::Derivative,
+  derive_where::derive_where,
   edit_distance::edit_distance,
   lexiclean::Lexiclean,
   libc::EXIT_FAILURE,
-  log::{info, warn},
   once_cell::sync::Lazy,
   regex::Regex,
   serde::{
