@@ -2,6 +2,11 @@ use {super::*, clap_mangen::Man};
 
 const INIT_JUSTFILE: &str = "default:\n    echo 'Hello, world!'\n";
 
+fn backtick_re() -> &'static Regex {
+  static BACKTICK_RE: OnceLock<Regex> = OnceLock::new();
+  BACKTICK_RE.get_or_init(|| Regex::new("(`.*?`)|(`[^`]*$)").unwrap())
+}
+
 #[derive(PartialEq, Clone, Debug)]
 pub(crate) enum Subcommand {
   Changelog,
@@ -53,10 +58,6 @@ impl Subcommand {
       Completions { shell } => return Self::completions(*shell),
       Init => return Self::init(config),
       Man => return Self::man(),
-      Run {
-        arguments,
-        overrides,
-      } => return Self::run(config, loader, arguments, overrides),
       _ => {}
     }
 
@@ -68,24 +69,26 @@ impl Subcommand {
 
     let compilation = Self::compile(config, loader, &search)?;
     let justfile = &compilation.justfile;
-    let ast = compilation.root_ast();
-    let src = compilation.root_src();
 
     match self {
+      Run {
+        arguments,
+        overrides,
+      } => Self::run(config, loader, search, compilation, arguments, overrides)?,
       Choose { overrides, chooser } => {
         Self::choose(config, justfile, &search, overrides, chooser.as_deref())?;
       }
       Command { overrides, .. } | Evaluate { overrides, .. } => {
         justfile.run(config, &search, overrides, &[])?;
       }
-      Dump => Self::dump(config, ast, justfile)?,
-      Format => Self::format(config, &search, src, ast, justfile)?,
+      Dump => Self::dump(config, compilation)?,
+      Format => Self::format(config, &search, compilation)?,
       Groups => Self::groups(config, justfile),
       List { path } => Self::list(config, justfile, path)?,
       Show { path } => Self::show(config, justfile, path)?,
       Summary => Self::summary(config, justfile),
       Variables => Self::variables(justfile),
-      Changelog | Completions { .. } | Edit | Init | Man | Run { .. } => unreachable!(),
+      Changelog | Completions { .. } | Edit | Init | Man => unreachable!(),
     }
 
     Ok(())
@@ -101,88 +104,49 @@ impl Subcommand {
   fn run<'src>(
     config: &Config,
     loader: &'src Loader,
+    mut search: Search,
+    mut compilation: Compilation<'src>,
     arguments: &[String],
     overrides: &BTreeMap<String, String>,
   ) -> RunResult<'src> {
-    if matches!(
-      config.search_config,
-      SearchConfig::FromInvocationDirectory | SearchConfig::FromSearchDirectory { .. }
-    ) {
-      let starting_path = match &config.search_config {
-        SearchConfig::FromInvocationDirectory => config.invocation_directory.clone(),
-        SearchConfig::FromSearchDirectory { search_directory } => config
-          .invocation_directory
-          .join(search_directory)
-          .lexiclean(),
-        _ => unreachable!(),
-      };
+    let starting_parent = search.justfile.parent().as_ref().unwrap().lexiclean();
 
-      let mut path = starting_path.clone();
+    loop {
+      let justfile = &compilation.justfile;
+      let fallback = justfile.settings.fallback
+        && matches!(
+          config.search_config,
+          SearchConfig::FromInvocationDirectory | SearchConfig::FromSearchDirectory { .. }
+        );
 
-      let mut unknown_recipes_errors = None;
+      let result = justfile.run(config, &search, overrides, arguments);
 
-      loop {
-        let search = match Search::find_next(&path) {
-          Err(SearchError::NotFound) => match unknown_recipes_errors {
-            Some(err) => return Err(err),
-            None => return Err(SearchError::NotFound.into()),
-          },
-          Err(err) => return Err(err.into()),
-          Ok(search) => {
-            if config.verbosity.loquacious() && path != starting_path {
-              eprintln!(
-                "Trying {}",
-                starting_path
-                  .strip_prefix(path)
-                  .unwrap()
-                  .components()
-                  .map(|_| path::Component::ParentDir)
-                  .collect::<PathBuf>()
-                  .join(search.justfile.file_name().unwrap())
-                  .display()
-              );
-            }
-            search
+      if fallback {
+        if let Err(err @ (Error::UnknownRecipe { .. } | Error::UnknownSubmodule { .. })) = result {
+          search = search.search_parent_directory().map_err(|_| err)?;
+
+          if config.verbosity.loquacious() {
+            eprintln!(
+              "Trying {}",
+              starting_parent
+                .strip_prefix(search.justfile.parent().unwrap())
+                .unwrap()
+                .components()
+                .map(|_| path::Component::ParentDir)
+                .collect::<PathBuf>()
+                .join(search.justfile.file_name().unwrap())
+                .display()
+            );
           }
-        };
 
-        match Self::run_inner(config, loader, arguments, overrides, &search) {
-          Err((err @ (Error::UnknownRecipe { .. } | Error::UnknownSubmodule { .. }), true)) => {
-            match search.justfile.parent().unwrap().parent() {
-              Some(parent) => {
-                unknown_recipes_errors.get_or_insert(err);
-                path = parent.into();
-              }
-              None => return Err(err),
-            }
-          }
-          result => return result.map_err(|(err, _fallback)| err),
+          compilation = Self::compile(config, loader, &search)?;
+
+          continue;
         }
       }
-    } else {
-      Self::run_inner(
-        config,
-        loader,
-        arguments,
-        overrides,
-        &Search::find(&config.search_config, &config.invocation_directory)?,
-      )
-      .map_err(|(err, _fallback)| err)
-    }
-  }
 
-  fn run_inner<'src>(
-    config: &Config,
-    loader: &'src Loader,
-    arguments: &[String],
-    overrides: &BTreeMap<String, String>,
-    search: &Search,
-  ) -> Result<(), (Error<'src>, bool)> {
-    let compilation = Self::compile(config, loader, search).map_err(|err| (err, false))?;
-    let justfile = &compilation.justfile;
-    justfile
-      .run(config, search, overrides, arguments)
-      .map_err(|err| (err, justfile.settings.fallback))
+      return result;
+    }
   }
 
   fn compile<'src>(
@@ -303,14 +267,14 @@ impl Subcommand {
     Ok(())
   }
 
-  fn dump(config: &Config, ast: &Ast, justfile: &Justfile) -> RunResult<'static> {
+  fn dump(config: &Config, compilation: Compilation) -> RunResult<'static> {
     match config.dump_format {
       DumpFormat::Json => {
-        serde_json::to_writer(io::stdout(), justfile)
+        serde_json::to_writer(io::stdout(), &compilation.justfile)
           .map_err(|serde_json_error| Error::DumpJson { serde_json_error })?;
         println!();
       }
-      DumpFormat::Just => print!("{ast}"),
+      DumpFormat::Just => print!("{}", compilation.root_ast()),
     }
     Ok(())
   }
@@ -337,13 +301,11 @@ impl Subcommand {
     Ok(())
   }
 
-  fn format(
-    config: &Config,
-    search: &Search,
-    src: &str,
-    ast: &Ast,
-    justfile: &Justfile,
-  ) -> RunResult<'static> {
+  fn format(config: &Config, search: &Search, compilation: Compilation) -> RunResult<'static> {
+    let justfile = &compilation.justfile;
+    let src = compilation.root_src();
+    let ast = compilation.root_ast();
+
     config.require_unstable(justfile, UnstableFeature::FormatSubcommand)?;
 
     let formatted = ast.to_string();
@@ -392,20 +354,23 @@ impl Subcommand {
     let search = Search::init(&config.search_config, &config.invocation_directory)?;
 
     if search.justfile.is_file() {
-      Err(Error::InitExists {
+      return Err(Error::InitExists {
         justfile: search.justfile,
-      })
-    } else if let Err(io_error) = fs::write(&search.justfile, INIT_JUSTFILE) {
-      Err(Error::WriteJustfile {
+      });
+    }
+
+    if let Err(io_error) = fs::write(&search.justfile, INIT_JUSTFILE) {
+      return Err(Error::WriteJustfile {
         justfile: search.justfile,
         io_error,
-      })
-    } else {
-      if config.verbosity.loud() {
-        eprintln!("Wrote justfile to `{}`", search.justfile.display());
-      }
-      Ok(())
+      });
     }
+
+    if config.verbosity.loud() {
+      eprintln!("Wrote justfile to `{}`", search.justfile.display());
+    }
+
+    Ok(())
   }
 
   fn man() -> RunResult<'static> {
@@ -453,15 +418,31 @@ impl Subcommand {
     ) {
       if let Some(doc) = doc {
         if !doc.is_empty() && doc.lines().count() <= 1 {
+          let color = config.color.stdout();
           print!(
-            "{:padding$}{} {}",
+            "{:padding$}{} ",
             "",
-            config.color.stdout().doc().paint("#"),
-            config.color.stdout().doc().paint(doc),
+            color.doc().paint("#"),
             padding = max_signature_width.saturating_sub(signature_widths[name]) + 1,
           );
+
+          let mut end = 0;
+          for backtick in backtick_re().find_iter(doc) {
+            let prefix = &doc[end..backtick.start()];
+            if !prefix.is_empty() {
+              print!("{}", color.doc().paint(prefix));
+            }
+            print!("{}", color.doc_backtick().paint(backtick.as_str()));
+            end = backtick.end();
+          }
+
+          let suffix = &doc[end..];
+          if !suffix.is_empty() {
+            print!("{}", color.doc().paint(suffix));
+          }
         }
       }
+
       println!();
     }
 
@@ -576,7 +557,10 @@ impl Subcommand {
 
       if !no_groups {
         if let Some(group) = &group {
-          println!("{list_prefix}[{group}]");
+          println!(
+            "{list_prefix}{}",
+            config.color.stdout().group().paint(&format!("[{group}]"))
+          );
         }
       }
 
