@@ -5,8 +5,8 @@ pub(crate) struct Variables<'expression, 'src> {
 }
 
 impl<'expression, 'src> Variables<'expression, 'src> {
-  pub(crate) fn new(root: &'expression Expression<'src>) -> Variables<'expression, 'src> {
-    Variables { stack: vec![root] }
+  pub(crate) fn new(root: &'expression Expression<'src>) -> Self {
+    Self { stack: vec![root] }
   }
 }
 
@@ -16,7 +16,24 @@ impl<'expression, 'src> Iterator for Variables<'expression, 'src> {
   fn next(&mut self) -> Option<Token<'src>> {
     loop {
       match self.stack.pop()? {
-        Expression::StringLiteral { .. } | Expression::Backtick { .. } => {}
+        Expression::And { lhs, rhs } | Expression::Or { lhs, rhs } => {
+          self.stack.push(lhs);
+          self.stack.push(rhs);
+        }
+        Expression::Assert {
+          condition:
+            Condition {
+              lhs,
+              rhs,
+              operator: _,
+            },
+          error,
+        } => {
+          self.stack.push(error);
+          self.stack.push(rhs);
+          self.stack.push(lhs);
+        }
+        Expression::Backtick { .. } | Expression::StringLiteral { .. } => {}
         Expression::Call { thunk } => match thunk {
           Thunk::Nullary { .. } => {}
           Thunk::Unary { arg, .. } => self.stack.push(arg),
@@ -28,6 +45,14 @@ impl<'expression, 'src> Iterator for Variables<'expression, 'src> {
               self.stack.push(b);
             }
           }
+          Thunk::UnaryPlus {
+            args: (a, rest), ..
+          } => {
+            let first: &[&Expression] = &[a];
+            for arg in first.iter().copied().chain(rest).rev() {
+              self.stack.push(arg);
+            }
+          }
           Thunk::Binary { args, .. } => {
             for arg in args.iter().rev() {
               self.stack.push(arg);
@@ -48,22 +73,27 @@ impl<'expression, 'src> Iterator for Variables<'expression, 'src> {
             }
           }
         },
+        Expression::Concatenation { lhs, rhs } => {
+          self.stack.push(rhs);
+          self.stack.push(lhs);
+        }
         Expression::Conditional {
-          lhs,
-          rhs,
+          condition:
+            Condition {
+              lhs,
+              rhs,
+              operator: _,
+            },
           then,
           otherwise,
-          ..
         } => {
           self.stack.push(otherwise);
           self.stack.push(then);
           self.stack.push(rhs);
           self.stack.push(lhs);
         }
-        Expression::Variable { name, .. } => return Some(name.token),
-        Expression::Concatenation { lhs, rhs } => {
-          self.stack.push(rhs);
-          self.stack.push(lhs);
+        Expression::Group { contents } => {
+          self.stack.push(contents);
         }
         Expression::Join { lhs, rhs } => {
           self.stack.push(rhs);
@@ -71,9 +101,7 @@ impl<'expression, 'src> Iterator for Variables<'expression, 'src> {
             self.stack.push(lhs);
           }
         }
-        Expression::Group { contents } => {
-          self.stack.push(contents);
-        }
+        Expression::Variable { name, .. } => return Some(name.token),
       }
     }
   }
