@@ -36,6 +36,7 @@ mod allow_duplicate_variables;
 mod assert_stdout;
 mod assert_success;
 mod assertions;
+mod assignment;
 mod attributes;
 mod backticks;
 mod byte_order_mark;
@@ -55,6 +56,7 @@ mod equals;
 mod error_messages;
 mod evaluate;
 mod examples;
+mod explain;
 mod export;
 mod fallback;
 mod fmt;
@@ -71,6 +73,7 @@ mod invocation_directory;
 mod json;
 mod line_prefixes;
 mod list;
+mod logical_operators;
 mod man;
 mod misc;
 mod modules;
@@ -81,6 +84,7 @@ mod no_cd;
 mod no_dependencies;
 mod no_exit_message;
 mod os_attributes;
+mod parameters;
 mod parser;
 mod positional_arguments;
 mod private;
