@@ -1,4 +1,7 @@
-use {super::*, pretty_assertions::assert_eq};
+use {
+  super::*,
+  pretty_assertions::{assert_eq, StrComparison},
+};
 
 macro_rules! test {
   {
@@ -150,7 +153,7 @@ impl Test {
   }
 
   pub(crate) fn stderr_regex(mut self, stderr_regex: impl AsRef<str>) -> Self {
-    self.stderr_regex = Some(Regex::new(&format!("^{}$", stderr_regex.as_ref())).unwrap());
+    self.stderr_regex = Some(Regex::new(&format!("^(?s){}$", stderr_regex.as_ref())).unwrap());
     self
   }
 
@@ -169,6 +172,7 @@ impl Test {
     self
   }
 
+  #[allow(unused)]
   pub(crate) fn test_round_trip(mut self, test_round_trip: bool) -> Self {
     self.test_round_trip = test_round_trip;
     self
@@ -196,6 +200,22 @@ impl Test {
 impl Test {
   #[track_caller]
   pub(crate) fn run(self) -> Output {
+    fn compare<T: PartialEq + Debug>(name: &str, have: T, want: T) -> bool {
+      let equal = have == want;
+      if !equal {
+        eprintln!("Bad {name}: {}", Comparison::new(&have, &want));
+      }
+      equal
+    }
+
+    fn compare_string(name: &str, have: &str, want: &str) -> bool {
+      let equal = have == want;
+      if !equal {
+        eprintln!("Bad {name}: {}", StrComparison::new(&have, &want));
+      }
+      equal
+    }
+
     if let Some(justfile) = &self.justfile {
       let justfile = unindent(justfile);
       fs::write(self.justfile_path(), justfile).unwrap();
@@ -239,32 +259,26 @@ impl Test {
       .wait_with_output()
       .expect("failed to wait for just process");
 
-    fn compare<T: PartialEq + Debug>(name: &str, have: T, want: T) -> bool {
-      let equal = have == want;
-      if !equal {
-        eprintln!("Bad {name}: {}", Comparison::new(&have, &want));
-      }
-      equal
-    }
-
     let output_stdout = str::from_utf8(&output.stdout).unwrap();
     let output_stderr = str::from_utf8(&output.stderr).unwrap();
 
     if let Some(ref stdout_regex) = self.stdout_regex {
-      if !stdout_regex.is_match(output_stdout) {
-        panic!("Stdout regex mismatch:\n{output_stdout:?}\n!~=\n/{stdout_regex:?}/");
-      }
+      assert!(
+        stdout_regex.is_match(output_stdout),
+        "Stdout regex mismatch:\n{output_stdout:?}\n!~=\n/{stdout_regex:?}/",
+      );
     }
 
     if let Some(ref stderr_regex) = self.stderr_regex {
-      if !stderr_regex.is_match(output_stderr) {
-        panic!("Stderr regex mismatch:\n{output_stderr:?}\n!~=\n/{stderr_regex:?}/");
-      }
+      assert!(
+        stderr_regex.is_match(output_stderr),
+        "Stderr regex mismatch:\n{output_stderr:?}\n!~=\n/{stderr_regex:?}/",
+      );
     }
 
     if !compare("status", output.status.code(), Some(self.status))
-      | (self.stdout_regex.is_none() && !compare("stdout", output_stdout, &stdout))
-      | (self.stderr_regex.is_none() && !compare("stderr", output_stderr, &stderr))
+      | (self.stdout_regex.is_none() && !compare_string("stdout", output_stdout, &stdout))
+      | (self.stderr_regex.is_none() && !compare_string("stderr", output_stderr, &stderr))
     {
       panic!("Output mismatch.");
     }
@@ -290,9 +304,12 @@ impl Test {
       .output()
       .expect("just invocation failed");
 
-    if !output.status.success() {
-      panic!("dump failed: {} {:?}", output.status, output);
-    }
+    assert!(
+      output.status.success(),
+      "dump failed: {} {:?}",
+      output.status,
+      output,
+    );
 
     let dumped = String::from_utf8(output.stdout).unwrap();
 
@@ -309,9 +326,7 @@ impl Test {
       .output()
       .expect("just invocation failed");
 
-    if !output.status.success() {
-      panic!("reparse failed: {}", output.status);
-    }
+    assert!(output.status.success(), "reparse failed: {}", output.status);
 
     let reparsed = String::from_utf8(output.stdout).unwrap();
 
